summaryrefslogtreecommitdiffstats
diff options
context:
space:
mode:
-rw-r--r--example/clientdata/equip.xml2
-rw-r--r--example/clientdata/graphics/items/equipment/weapons/weapon-dagger-dagger.pngbin0 -> 582 bytes
-rw-r--r--example/clientdata/graphics/sprites/weapons/weapon-dagger.xml27
-rw-r--r--example/clientdata/items.xml17
-rw-r--r--example/serverdata/scripts/maps/desert.lua2
-rw-r--r--src/common/inventorydata.h2
-rw-r--r--src/game-server/inventory.cpp225
-rw-r--r--src/game-server/inventory.h8
-rw-r--r--src/game-server/item.h25
-rw-r--r--src/game-server/itemmanager.cpp21
10 files changed, 194 insertions, 135 deletions
diff --git a/example/clientdata/equip.xml b/example/clientdata/equip.xml
index 097229c..1a17fd2 100644
--- a/example/clientdata/equip.xml
+++ b/example/clientdata/equip.xml
@@ -14,6 +14,6 @@
<slot id="5" name="legs" capacity="1" visible="true" />
<slot id="6" name="ring" capacity="1" />
<slot id="7" name="necklace" capacity="1" />
- <slot id="8" name="feet" capacity="1" />
+ <slot id="8" name="feet" capacity="1" visible="true" />
<slot id="9" name="ammo" capacity="1" />
</equip-slots>
diff --git a/example/clientdata/graphics/items/equipment/weapons/weapon-dagger-dagger.png b/example/clientdata/graphics/items/equipment/weapons/weapon-dagger-dagger.png
new file mode 100644
index 0000000..cdfc370
--- /dev/null
+++ b/example/clientdata/graphics/items/equipment/weapons/weapon-dagger-dagger.png
Binary files differ
diff --git a/example/clientdata/graphics/sprites/weapons/weapon-dagger.xml b/example/clientdata/graphics/sprites/weapons/weapon-dagger.xml
new file mode 100644
index 0000000..1909aff
--- /dev/null
+++ b/example/clientdata/graphics/sprites/weapons/weapon-dagger.xml
@@ -0,0 +1,27 @@
+<?xml version="1.0"?>
+<sprite name="player" action="stand">
+ <imageset name="dagger" src="graphics/sprites/weapons/weapon-sword.png" width="64" height="64" />
+
+ <action name="default" imageset="dagger">
+ </action>
+
+ <action name="attack_stab" imageset="dagger">
+ <animation direction="down">
+ <sequence start="0" end="3" delay="75" />
+ <end />
+ </animation>
+ <animation direction="left">
+ <sequence start="4" end="7" delay="75" />
+ <end />
+ </animation>
+ <animation direction="up">
+ <sequence start="8" end="11" delay="75" />
+ <end />
+ </animation>
+ <animation direction="right">
+ <sequence start="12" end="15" delay="75" />
+ <end />
+ </animation>
+ </action>
+
+</sprite> \ No newline at end of file
diff --git a/example/clientdata/items.xml b/example/clientdata/items.xml
index 6f72185..3e204ef 100644
--- a/example/clientdata/items.xml
+++ b/example/clientdata/items.xml
@@ -142,6 +142,23 @@
</effect>
<sprite>weapons/weapon-bow.xml</sprite>
</item>
+ <item id="10" max-per-slot="3" name="Dagger" attack-action="attack_stab"
+ description="A simple dagger."
+ image="equipment/weapons/weapon-dagger-dagger.png"
+ value="20">
+ <equip>
+ <slot type="hand" required="1" />
+ </equip>
+ <effect trigger="equip">
+ <autoattack basespeed="166" range="1" base="50" skill="100" >
+ <bonus attribute="0" value="1.5" />
+ </autoattack>
+ <effect trigger="existence">
+ <modifier attribute="cap1" value="-2000" />
+ </effect>
+ </effect>
+ <sprite>weapons/weapon-dagger.xml</sprite>
+ </item>
<!-- Example of equippable items -->
<item id="7" max-per-slot="1" name="Leather shirt"
diff --git a/example/serverdata/scripts/maps/desert.lua b/example/serverdata/scripts/maps/desert.lua
index b2eb9a4..e6a12d1 100644
--- a/example/serverdata/scripts/maps/desert.lua
+++ b/example/serverdata/scripts/maps/desert.lua
@@ -33,7 +33,7 @@ atinit(function()
create_npc("Merchant", 3, 4 * TILESIZE + TILESIZE / 2, 16 * TILESIZE + TILESIZE / 2, npclib.talk(Merchant, merchant_buy_table, merchant_sell_table), nil)
-- Another Merchant, selling some equipment, and buying everything...
- smith_buy_table = { {5, 10, 50}, {7, 10, 70} }
+ smith_buy_table = { {5, 10, 50}, {7, 10, 70}, {10, 10, 20} }
create_npc("Smith", 5, 15 * TILESIZE + TILESIZE / 2, 16 * TILESIZE + TILESIZE / 2, npclib.talk(Merchant, smith_buy_table), nil)
-- The most simple NPC - Welcoming new ones around.
diff --git a/src/common/inventorydata.h b/src/common/inventorydata.h
index e1c5bfa..490abff 100644
--- a/src/common/inventorydata.h
+++ b/src/common/inventorydata.h
@@ -65,7 +65,7 @@ struct EquipmentItem
// inventory slot id -> { item }
typedef std::map< unsigned int, InventoryItem > InventoryData;
-// equip slot id -> { item id }
+// equip slot id -> { item id, item instance }
// Equipment taking up multiple equip slot ids will be referenced multiple times
typedef std::multimap< unsigned int, EquipmentItem > EquipData;
diff --git a/src/game-server/inventory.cpp b/src/game-server/inventory.cpp
index c3d2bfc..5598610 100644
--- a/src/game-server/inventory.cpp
+++ b/src/game-server/inventory.cpp
@@ -600,58 +600,66 @@ bool Inventory::equip(int inventorySlot)
return false;
// Test the equip requirements. If none, it's not an equipable item.
- const ItemEquipsInfo &equipInfoList =
- itemManager->getItem(it->second.itemId)->getItemEquipData();
- if (equipInfoList.empty())
+ const ItemEquipRequirement &equipReq =
+ itemManager->getItem(it->second.itemId)->getItemEquipRequirement();
+ if (!equipReq.equipSlotId)
{
LOG_DEBUG("No equip requirements for item id: " << it->second.itemId
<< " at slot: " << inventorySlot);
return false;
}
- // Iterate through all slots requirements.
- std::list<unsigned int> equipSlotsToUnequipFirst;
- for (ItemEquipsInfo::const_iterator it2 = equipInfoList.begin(),
- it2_end = equipInfoList.end(); it2 != it2_end; ++it2)
- {
- // We first check the equipment slots for:
- // - 1. whether enough total equip slot space is available.
- // - 2. whether some other equipment is to be unequipped first.
+ // List of potential unique itemInstances to unequip first.
+ std::set<unsigned int> equipInstancesToUnequipFirst;
- // If not enough total space in the equipment slot is available,
- // we cannot equip.
- if (itemManager->getEquipSlotCapacity(it2->first) < it2->second)
- {
- LOG_DEBUG("Not enough equip capacity at slot: " << it2->first
- << ", total available: "
- << itemManager->getEquipSlotCapacity(it2->first)
- << ", required: " << it2->second);
- return false;
- }
+ // We first check the equipment slots for:
+ // - 1. whether enough total equip slot space is available.
+ // - 2. whether some other equipment is to be unequipped first.
+
+ // If not enough total space in the equipment slot is available,
+ // we cannot equip.
+ if (itemManager->getEquipSlotCapacity(equipReq.equipSlotId)
+ < equipReq.capacityRequired)
+ {
+ LOG_DEBUG("Not enough equip capacity at slot: " << equipReq.equipSlotId
+ << ", total available: "
+ << itemManager->getEquipSlotCapacity(equipReq.equipSlotId)
+ << ", required: " << equipReq.capacityRequired);
+ return false;
+ }
- // Test whether some item(s) is(are) to be unequipped first.
- if (!checkEquipmentCapacity(it2->first, it2->second))
+ // Test whether some item(s) is(are) to be unequipped first.
+ if (!checkEquipmentCapacity(equipReq.equipSlotId,
+ equipReq.capacityRequired))
+ {
+ // And test whether the unequip action would succeed first.
+ if (testUnequipScriptRequirements(equipReq.equipSlotId)
+ && hasInventoryEnoughSpace(equipReq.equipSlotId))
{
- // And test whether the unequip action would succeed first.
- if (testUnequipScriptRequirements(it2->first)
- && hasInventoryEnoughSpace(it2->first))
+ // Then, we unequip each iteminstance of the equip slot
+ for (EquipData::iterator iter =
+ mPoss->equipSlots.begin();
+ iter != mPoss->equipSlots.end(); ++iter)
{
- equipSlotsToUnequipFirst.push_back(it2->first);
- }
- else
- {
- // Some non-unequippable equipment is to be unequipped first.
- // Can be the case of cursed items,
- // or when the inventory is full, for instance.
- return false;
+ if (iter->first == equipReq.equipSlotId
+ && iter->second.itemInstance)
+ equipInstancesToUnequipFirst.insert(
+ iter->second.itemInstance);
}
}
+ else
+ {
+ // Some non-unequippable equipment is to be unequipped first.
+ // Can be the case of cursed items,
+ // or when the inventory is full, for instance.
+ return false;
+ }
}
// Potential Pre-unequipment process
- for (std::list<unsigned int>::const_iterator it3 =
- equipSlotsToUnequipFirst.begin();
- it3 != equipSlotsToUnequipFirst.end(); ++it3)
+ for (std::set<unsigned int>::const_iterator it3 =
+ equipInstancesToUnequipFirst.begin();
+ it3 != equipInstancesToUnequipFirst.end(); ++it3)
{
if (!unequip(*it3))
{
@@ -667,69 +675,65 @@ bool Inventory::equip(int inventorySlot)
//W equip slot type count, W item id, { W equip slot, W capacity used}*
MessageOut equipMsg(GPMSG_EQUIP);
equipMsg.writeInt16(it->second.itemId); // Item Id
- equipMsg.writeInt16(equipInfoList.size()); // Number of equip slot changed.
+ equipMsg.writeInt16(1); // Number of equip slot changed.
// Compute an unique equip item Instance id (unicity is per character only.)
int itemInstance = getNewEquipItemInstance();
- for (ItemEquipsInfo::const_iterator it2 = equipInfoList.begin(),
- it2_end = equipInfoList.end(); it2 != it2_end; ++it2)
+ unsigned int capacityLeft = equipReq.capacityRequired;
+ unsigned int capacityUsed = 0;
+ // Apply equipment changes
+ for (EquipData::iterator it4 = mPoss->equipSlots.begin(),
+ it4_end = mPoss->equipSlots.end(); it4 != it4_end; ++it4)
{
- unsigned int capacityLeft = it2->second;
- unsigned int capacityUsed = 0;
- // Apply equipment changes
- for (EquipData::iterator it4 = mPoss->equipSlots.begin(),
- it4_end = mPoss->equipSlots.end(); it4 != it4_end; ++it4)
- {
- if (!capacityLeft)
- break;
+ if (!capacityLeft)
+ break;
- // We've found an existing equip slot
- if (it4->first == it2->first)
+ // We've found an existing equip slot
+ if (it4->first == equipReq.equipSlotId)
+ {
+ // We've found an empty slot
+ if (it4->second.itemInstance == 0)
{
- // We've found an empty slot
- if (it4->second.itemInstance == 0)
- {
- it4->second.itemId = it->second.itemId;
- it4->second.itemInstance = itemInstance;
- --capacityLeft;
- }
- else // The slot is already in use.
- {
- ++capacityUsed;
- }
+ it4->second.itemId = it->second.itemId;
+ it4->second.itemInstance = itemInstance;
+ --capacityLeft;
+ }
+ else // The slot is already in use.
+ {
+ ++capacityUsed;
}
}
+ }
- // When there is still something to apply even when out of that loop,
- // It means that the equip multimapis missing empty slots.
- // Hence, we add them back
- if(capacityLeft)
- {
- unsigned int maxCapacity =
- itemManager->getEquipSlotCapacity(it2->first);
+ // When there is still something to apply even when out of that loop,
+ // It means that the equip multimapis missing empty slots.
+ // Hence, we add them back
+ if(capacityLeft)
+ {
+ unsigned int maxCapacity =
+ itemManager->getEquipSlotCapacity(equipReq.equipSlotId);
- // A should never happen case
- assert(maxCapacity >= capacityUsed + capacityLeft);
+ // A should never happen case
+ assert(maxCapacity >= capacityUsed + capacityLeft);
- while (capacityLeft)
- {
- EquipmentItem equipItem(it->second.itemId, itemInstance);
- mPoss->equipSlots.insert(
- std::make_pair<unsigned int, EquipmentItem>
- (it2->first, equipItem));
- --capacityLeft;
- }
+ while (capacityLeft)
+ {
+ EquipmentItem equipItem(it->second.itemId, itemInstance);
+ mPoss->equipSlots.insert(
+ std::make_pair<unsigned int, EquipmentItem>
+ (equipReq.equipSlotId, equipItem));
+ --capacityLeft;
}
-
- // Equip slot
- equipMsg.writeInt16(it2->first);
- // Capacity used
- equipMsg.writeInt16(it2->second);
- // Item instance
- equipMsg.writeInt16(itemInstance);
}
+ // Equip slot
+ equipMsg.writeInt16(equipReq.equipSlotId);
+ // Capacity used
+ equipMsg.writeInt16(equipReq.capacityRequired);
+ // Item instance
+ equipMsg.writeInt16(itemInstance);
+
// New item trigger
updateEquipmentTrigger(0, it->second.itemId);
@@ -744,54 +748,47 @@ bool Inventory::equip(int inventorySlot)
bool Inventory::unequip(unsigned int itemInstance)
{
- // map of { itemInstance, itemId }
- std::map<unsigned int, unsigned int> itemIdListToInventory;
+ if (!itemInstance)
+ return false;
MessageOut equipMsg(GPMSG_EQUIP);
equipMsg.writeInt16(0); // Item Id, useless in case of unequip.
+ // The itemId to unequip
+ unsigned int itemId = 0;
+
// Empties all equip entries that point to the given equipment slot
// The equipment slots should NEVER be erased after initialization!
for (EquipData::iterator it = mPoss->equipSlots.begin(),
it_end = mPoss->equipSlots.end(); it != it_end; ++it)
{
- if (it->second.itemInstance == itemInstance)
+ if (it->second.itemInstance == itemInstance && it->second.itemId)
{
// Add the item to the inventory list if not already present there
- std::map<unsigned int, unsigned int>::const_iterator it2 =
- itemIdListToInventory.find(it->second.itemInstance);
- if (it2 == itemIdListToInventory.end())
- {
- itemIdListToInventory.insert(
- std::make_pair<unsigned int, unsigned int>
- (it->second.itemInstance, it->second.itemId));
- }
-
+ itemId = it->second.itemId;
it->second.itemId = 0;
it->second.itemInstance = 0;
}
}
+ // When there were no corresponding item id, it means no item was to
+ // be unequipped.
+ if (!itemId)
+ return false;
+
// Number of slot types touched,
- equipMsg.writeInt16(itemIdListToInventory.size());
+ equipMsg.writeInt16(1);
- // Apply unequip trigger(s), and move the item(s) back to inventory.
- for (std::map<unsigned int, unsigned int>::const_iterator it2 =
- itemIdListToInventory.begin(), it2_end = itemIdListToInventory.end();
- it2 != it2_end; ++it2)
- {
- updateEquipmentTrigger(it2->second, 0);
- insert(it2->second, 1);
+ // Move the item back to inventory.
+ insert(itemId, 1);
- equipMsg.writeInt16(it2->first);
- equipMsg.writeInt16(0); // Capacity used, set to 0 to unequip.
- }
+ equipMsg.writeInt16(itemInstance);
+ equipMsg.writeInt16(0); // Capacity used, set to 0 to unequip.
- if (equipMsg.getLength() > 2)
- {
- gameHandler->sendTo(mCharacter, equipMsg);
- return true;
- }
+ gameHandler->sendTo(mCharacter, equipMsg);
+
+ // Apply unequip trigger
+ updateEquipmentTrigger(itemId, 0);
- return false;
+ return true;
}
diff --git a/src/game-server/inventory.h b/src/game-server/inventory.h
index 39f8321..1eaf052 100644
--- a/src/game-server/inventory.h
+++ b/src/game-server/inventory.h
@@ -120,7 +120,7 @@ class Inventory
* @todo
*/
bool hasInventoryEnoughSpace(unsigned int equipmentSlot)
- { return false; }
+ { return true; }
/**
* Test the items unequipment requirements.
@@ -153,6 +153,12 @@ class Inventory
void checkInventorySize();
/**
+ * Check whether the equipment change has visible consequence
+ * on the character sprite.
+ */
+ void checkLookUpdate(unsigned slotTypeId);
+
+ /**
* Apply equipment triggers.
*/
void updateEquipmentTrigger(unsigned int oldId, unsigned int itemId);
diff --git a/src/game-server/itemmanager.cpp b/src/game-server/itemmanager.cpp
index 074b834..d41212b 100644
--- a/src/game-server/itemmanager.cpp
+++ b/src/game-server/itemmanager.cpp
@@ -264,6 +264,13 @@ void ItemManager::readEquipNode(xmlNodePtr equipNode, ItemClass *item)
{
if (xmlStrEqual(subNode->name, BAD_CAST "slot"))
{
+ if (item->mEquipReq.equipSlotId)
+ {
+ LOG_WARN("Item Manager: duplicate equip slot definitions!"
+ " Only the first will apply.");
+ break;
+ }
+
std::string slot = XML::getProperty(subNode, "type", std::string());
if (slot.empty())
{
@@ -273,21 +280,23 @@ void ItemManager::readEquipNode(xmlNodePtr equipNode, ItemClass *item)
if (utils::isNumeric(slot))
{
// When the slot id is given
- item->mEquip.push_back(std::make_pair(utils::stringToInt(slot),
- XML::getProperty(subNode, "required", 1)));
+ item->mEquipReq.equipSlotId = utils::stringToInt(slot);
}
else
{
// When its name is given
- item->mEquip.push_back(std::make_pair(getEquipSlotIdFromName(slot),
- XML::getProperty(subNode, "required", 1)));
+ item->mEquipReq.equipSlotId = getEquipSlotIdFromName(slot);
}
+ item->mEquipReq.capacityRequired =
+ XML::getProperty(subNode, "required", 1);
}
}
- if (item->mEquip.empty())
+
+ if (!item->mEquipReq.equipSlotId)
{
LOG_WARN("Item Manager: empty equip requirement "
- "definition for item " << item->getDatabaseID() << "!");
+ "definition for item " << item->getDatabaseID() << "!"
+ " The item will be unequippable.");
return;
}
}
diff --git a/src/game-server/item.h b/src/game-server/item.h
index caef386..f7c380f 100644
--- a/src/game-server/item.h
+++ b/src/game-server/item.h
@@ -28,10 +28,15 @@
class Being;
class Script;
-// A pair indicating: Equipment slot id -> how much slots required.
-typedef std::pair< unsigned int, unsigned int> ItemEquipInfo;
-// The list of required slots to equip.
-typedef std::list< ItemEquipInfo > ItemEquipsInfo;
+// Indicates the equip slot "cost" to equip an item.
+struct ItemEquipRequirement {
+ ItemEquipRequirement():
+ equipSlotId(0),
+ capacityRequired(0)
+ {}
+
+ unsigned int equipSlotId, capacityRequired;
+};
/**
* State effects to beings, and actors.
@@ -226,9 +231,10 @@ class ItemClass
{ return mSpriteID; }
/**
* Returns equip requirements.
*/
- const ItemEquipsInfo &getItemEquipData() const { return mEquip; }
+ const ItemEquipRequirement &getItemEquipRequirement() const
+ { return mEquipReq; }
private:
/**
@@ -274,12 +280,9 @@ class ItemClass
std::multimap< ItemTriggerType, ItemEffectInfo * > mDispells;
/**
- * List of list of requirements for equipping. Only one inner list
- * need be satisfied to sucessfully equip. Checks occur in order
- * from outer front to back.
- * All conditions in an inner list must be met for success.
+ * Requirement for equipping.
*/
- ItemEquipsInfo mEquip;
+ ItemEquipRequirement mEquipReq;
friend class ItemManager;
};
@@ -226,9 +231,10 @@ class ItemClass
{ return mSpriteID; }
/**
- * Returns equip requirements.
+ * Returns equip requirement.
*/